      SUBROUTINE ice_gas_solu

!------------------------------------------------------------------------------!
!                               *** ice_gas_solu ***
!
!     This routine computes for Argon, Oxygen, CO2 and Nitrogen
!     -  Saturation concentration (csat_gas, mmol/m3)
!     -  Solubility (sol_gas, mmol/m3/atm)
!     in each vertical layer
!
!     Refs
!        Argon: Hamme and Emerson (2004)
!        Oxygen: Garcia and Gordon (1992)
!        CO2: Sarmiento and Gruber (2006), Weiss (1974)
!        N2: Sarmiento and Gruber (2006) and references therein
!     
!     Original code (in ice_gas.f): S. Moreau, M. Vancoppenolle, 2012-2015
!
!     Rewriting: M. Vancopppenolle, Nov 2015
!
!     Present status : 
!        - code for Ar, not for N2 and O2
!        - should put mixing ratios in the namelist
!------------------------------------------------------------------------------!

      USE lib_fortran

      INCLUDE 'type.com'
      INCLUDE 'para.com'
      INCLUDE 'const.com'
      INCLUDE 'ice.com'
      INCLUDE 'thermo.com'
      INCLUDE 'bio.com'

      REAL(8), DIMENSION(nlay_bio) ::  
     &   ztc,                      ! Celsius temperature
     &   ztlog                     ! Re-scaled temperature

       LOGICAL ::
     &   ln_write_gas = .TRUE.
      
!==============================================================================!

!
!------------------------------------------------------------------------------!
! X) Starting the routine
!------------------------------------------------------------------------------!
!

      IF ( ln_write_gas ) THEN

         WRITE(numout,*)
         WRITE(numout,*) ' *** ice_gas_solu : '
         WRITE(numout,*) ' ~~~~~~~~~~~~~~~~~~~'
         WRITE(numout,*)

      ENDIF
     
      ji = 1
      zpatm      = psbqb(ji) / 101325.                         ! atmospheric pressure in atmospheres

      CALL ice_brine
      
      IF ( ln_write_gas ) THEN
         WRITE(numout,*) ' ztc   : ', ztc(1:nlay_bio)
         WRITE(numout,*) ' sbr_bio  : ', sbr_bio(1:nlay_bio)
         WRITE(numout,*) ' rhobr_bio: ', rhobr_bio(1:nlay_bio)
         WRITE(numout,*)
      ENDIF

      DO jn = 1, ntra_bio

         IF ( flag_active(jn) .AND. ( biotr_i_typ(jn) .EQ. 'gas' ) ) 
     &   THEN 

!
!------------------------------------------------------------------------------!
! X) Celsius temperature and rescaled temperature
!------------------------------------------------------------------------------!
!
         
         DO layer = 1, nlay_bio
         
            ztc(layer)     = tc_bio(layer)                            ! temperature in celsius
         
            ztlog(layer)  = LOG( ( 298.15 - ztc(layer) ) /            ! rescaled log temperature
     &                           ( 273.15 + ztc(layer) ) )
            
         END DO

!        
!------------------------------------------------------------------------------!
! X) Saturation concentrations
!------------------------------------------------------------------------------!
!                                                   !-------
         IF ( biotr_i_nam(jn) .EQ. 'Arg' ) THEN    ! Argon
                                                   !-------

           IF ( ln_write_gas ) THEN
              WRITE(numout,*) ' --- Argon ...'
           ENDIF

           ! Hamme and Emerson, 2004
           ! form ln C = AO + A1 * Ts + A2 * Ts^2 + A3 * Ts^3
           !           + S ( B0 + B1 * Ts + B2 * Ts^2 )
           ! Ts = f(Tc), Tc in Celsius, S in pss

           !--- Polynomial coefficients (mumol/kg, Table 4)
           za0 = 2.79150     ; za1 = 3.17609     ; za2 = 4.13116
           za3 = 4.90379
           zb0 = -6.96233e-3 ; zb1 = -7.66670e-3 ; zb2 = -1.16888e-2

           DO layer = 1, nlay_bio

              !--- Rescaled log temperature powers
              zt  = ztlog(layer)
              zt2 = zt * zt
              zt3 = zt * zt2
               
              !--- Saturation concentration at equilibrium with a moist atmosphere at 1-atm pressure
              zln_csat = za0 + za1*zt + za2*zt2 + za3*zt3 +
     &                 ( zb0 + zb1*zt + zb2*zt2 ) * sbr_bio(layer)
     
              csat_gas(jn,layer) = EXP( zln_csat )                     ! in mumol/kg

              csat_gas(jn,layer) = csat_gas(jn,layer) *                ! in mmol/m3
     &                             rhobr_bio(layer) / 1000. 
     
           END DO
           
         ENDIF     ! 'Arg'
         
                                                   !-----------
         IF ( biotr_i_nam(jn) .EQ. 'Oxy' ) THEN    ! di-Oxygen
                                                   !-----------
         
           IF ( ln_write_gas ) THEN
              WRITE(numout,*) ' --- Oxygen ...'
           ENDIF
        
           ! Garcia and Gordon, L&O 1992
           ! form ln C = f(S,T), eq 8 page 1310 (! small mistake in the equation, extra-term)

           !--- Polynomial coefficients (mumol/kg, Table 1, last column)
           za0_sat_oxy = 5.80818;   za1_sat_oxy = 3.20684
           za2_sat_oxy = 4.11890;   za3_sat_oxy = 4.93845
           za4_sat_oxy = 1.01567;   za5_sat_oxy = 1.41575
           zb0_sat_oxy = -7.01211e-3;   zb1_sat_oxy = -7.25958e-3
           zb2_sat_oxy = -7.933343e-3;  zb3_sat_oxy = -5.54491e-3
           zc0_sat_oxy = -1.32412e-7

           DO layer = 1, nlay_bio
           
               !--- Rescaled log temperature powers
               zt   = ztlog(layer)   
               zt2  = zt * zt
               zt3  = zt2 * zt
               zt4  = zt2 * zt2
               zt5  = zt2 * zt3

               !--- Saturation concentration at equilibrium with a moist atmosphere at 1-atm pressure
               zln_csat = ( za0_sat_oxy + za1_sat_oxy * zt + 
     &                      za2_sat_oxy * zt2 +             
     &                      za3_sat_oxy * zt3 +
     &                      za4_sat_oxy * zt4 +
     &                      za5_sat_oxy * zt5 +
     &                      sbr_bio(layer) * 
     &                      ( zb0_sat_oxy + zb1_sat_oxy * zt +
     &                      zb2_sat_oxy * zt2 +
     &                      zb3_sat_oxy * zt3 ) +
     &                      zc0_sat_oxy *
     &                      sbr_bio(layer) * sbr_bio(layer) )
     
               csat_gas(jn,layer) = EXP(zln_csat)                   ! mumol/kg
               
               csat_gas(jn,layer) = csat_gas(jn,layer) *            ! in mmol/m3
     &                              rhobr_bio(layer) / 1000. 

           END DO
            
         ENDIF     ! 'Oxy'
         
                                                   !-------------
         IF ( biotr_i_nam(jn) .EQ. 'Nit' ) THEN    ! di-Nitrogen
                                                   !-------------
    
            IF ( ln_write_gas ) THEN
               WRITE(numout,*) ' --- Nitrogen ...'
            ENDIF                                           
                                                   
            ! Sarmiento and Gruber (2006, page 74) and references therein
            ! form ...

            !--- Polynomial coefficients (mumol/kg)
            za0_sat_nit = 6.42931
            za1_sat_nit = 2.92704
            za2_sat_nit = 4.32531
            za3_sat_nit = 4.69149
            zb0_sat_nit = -0.00744129
            zb1_sat_nit = -0.00802566
            zb2_sat_nit = -0.0146775

            DO layer = 1, nlay_bio

               !--- Rescaled log temperature powers
               zts  = ztlog(layer) 
               zts2 = zts * zts
               zts3 = zts2 * zts

               zln_csat = za0_sat_nit + za1_sat_nit * zts +  
     &                    za2_sat_nit * zts2 +                   
     &                    za3_sat_nit * zts3                     
     &                  + sbr_bio(layer) *
     &                  ( zb0_sat_nit + zb1_sat_nit * zts +
     &                    zb2_sat_nit * zts2 )
     
               csat_gas(jn,layer) = EXP(zln_csat)                   ! mumol/kg
                                                      
               csat_gas(jn,layer) = csat_gas(jn,layer) *            ! in mmol/m3
     &                              rhobr_bio(layer) / 1000. 

            END DO ! layer

         ENDIF     ! 'Nit'
         
                                                   !-------------
         IF ( biotr_i_nam(jn) .EQ. 'CO2' ) THEN    ! CO2
                                                   !-------------
                                                   
            IF ( ln_write_gas ) THEN
               WRITE(numout,*) ' --- CO2 ...'
            ENDIF                                           

           ! Sarmiento and Gruber, 2006 (Weiss, 1974 with total ph scale)
           ! form C = mix_rat * exp[ f(S,T) ], Table 3.2.2

           za0_sat_CO2 = -160.7333;   za1_sat_CO2 = 215.4152
           za2_sat_CO2 = 89.8920;     za3_sat_CO2 = -1.47759

           zb0_sat_CO2 = 0.029941;    zb1_sat_CO2 = -0.027455
           zb2_sat_CO2 = 0.0053407

           DO layer = 1, nlay_bio
        
               !--- temperature powers
               zt     = t_i_bio(layer)                             ! Temperature (K)
               z100_t = 100. / zt
               zt_100 = zt / 100.
               zt_100_2 = zt_100 * zt_100

               zsat_CO2 = mixr_gas(jn) *
     &                  EXP ( za0_sat_CO2 +       
     &                        za1_sat_CO2 * z100_t +           ! zsat_CO2 is the CO2 concentration at saturation
     &                        za2_sat_CO2 * log (zt_100) +     ! zsat_CO2 is in mol/kg
     &                        za3_sat_CO2 *  zt_100_2    +
     &                        sbr_bio(layer) *
     &                      ( zb0_sat_CO2 + 
     &                        zb1_sat_CO2 * zt_100 +
     &                        zb2_sat_CO2 * zt_100_2 ) )
     
               csat_gas(jn,layer) = zsat_CO2 * rhobr_bio(layer) * 1000. ! mmol/m3

!              WRITE(numout,*) '--------------------------- '
!              WRITE(numout,*) ' jn, layer: ', jn,layer
!              WRITE(numout,*) ' mixr_gas : ', mixr_gas(jn)
!              WRITE(numout,*) ' rhobr_bio: ', rhobr_bio(layer)
!              WRITE(numout,*) ' zsat_CO2 : ', zsat_CO2
!              WRITE(numout,*) ' zt       : ', zt
!              WRITE(numout,*) ' z100_t   : ', z100_t
!              WRITE(numout,*) ' zt_100   : ', zt_100
!              WRITE(numout,*) ' zt_100_2 : ', zt_100_2
!              WRITE(numout,*) ' csat_gas : ', csat_gas(jn,layer)
!              WRITE(numout,*) '--------------------------- '

            END DO ! layer
            
         ENDIF     ! 'CO2'

         WRITE(numout,*) ' ---> csat_gas: ', csat_gas(jn,:)
         WRITE(numout,*)

!------------------------------------------------------------------------------!

         ENDIF ! flags
      END DO ! jn
      
!
!------------------------------------------------------------------------------!
! X) Solubility and partial pressure
!------------------------------------------------------------------------------!
!
      DO jn = 1, ntra_bio
      
         sol_gas(jn,:) = 0.

         IF ( flag_active(jn) .AND. ( biotr_i_typ(jn) .EQ. 'gas' ) ) 
     &   THEN 

            sol_gas(jn,:) = csat_gas(jn,:) / mixr_gas(jn) ! solubility, ./m3/atm

            IF ( ln_write_gas ) THEN
               WRITE(numout,*) ' Tracer number : ', jn
               WRITE(numout,*) ' --- ', biotr_i_nam(jn)
               WRITE(numout,*) ' sol_gas(jn,:) : ', sol_gas(jn,:)
            ENDIF
         
         ENDIF

      END DO
!
!------------------------------------------------------------------------------!
! X) End of the routine
!------------------------------------------------------------------------------!
!
      WRITE(numout,*)
      WRITE(numout,*) ' End of ice_gas_solu '
      WRITE(numout,*) '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
      WRITE(numout,*)
     
      RETURN

      END
